@@ -236,6 +236,8 @@ struct ath5k_srev_name {
 #define AR5K_SREV_VER_AR5416	0xc0 /* PCI-E */
 #define AR5K_SREV_VER_AR5418	0xca /* PCI-E */
 #define AR5K_SREV_VER_AR2425	0xe2 /* PCI-E */
+#define AR5K_SREV_VER_AR9160	0x400
+#define AR5K_SREV_VER_AR9280	0x800
 
 #define AR5K_SREV_RAD_5110	0x00
 #define AR5K_SREV_RAD_5111	0x10
@@ -268,7 +270,7 @@ enum ath5k_dmasize {
 
 
 int ath_set_ack_bitrate(struct ath_softc *sc, int);
-int ar_device(int devid);
+int ar_device(struct ath_softc *sc);
 const char * ath5k_chip_name(enum ath5k_srev_type type, u_int16_t val);
 void ath_hw_beacon_stop(struct ath_softc *sc);
 int ath_hw_check_atim(struct ath_softc *sc, int window, int intval);
