@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_ioctl.h 3497 2008-04-09 22:25:51Z mtaylor $
+ * $Id: ieee80211_ioctl.h 4008 2009-04-22 22:05:57Z proski $
  */
 #ifndef _NET80211_IEEE80211_IOCTL_H_
 #define _NET80211_IEEE80211_IOCTL_H_
@@ -539,7 +539,6 @@ struct ieee80211req_scan_result {
 #define	IEEE80211_IOCTL_CHANSWITCH	(SIOCIWFIRSTPRIV+8)
 #define	IEEE80211_IOCTL_GET_APPIEBUF	(SIOCIWFIRSTPRIV+9)
 #define	IEEE80211_IOCTL_SET_APPIEBUF	(SIOCIWFIRSTPRIV+10)
-#define	IEEE80211_IOCTL_READREG		(SIOCIWFIRSTPRIV+11)
 #define	IEEE80211_IOCTL_FILTERFRAME	(SIOCIWFIRSTPRIV+12)
 #define	IEEE80211_IOCTL_GETCHANINFO	(SIOCIWFIRSTPRIV+13)
 #define	IEEE80211_IOCTL_SETOPTIE	(SIOCIWFIRSTPRIV+14)
@@ -547,7 +546,6 @@ struct ieee80211req_scan_result {
 #define	IEEE80211_IOCTL_SETMLME		(SIOCIWFIRSTPRIV+16)
 #define	IEEE80211_IOCTL_RADAR		(SIOCIWFIRSTPRIV+17)
 #define	IEEE80211_IOCTL_SETKEY		(SIOCIWFIRSTPRIV+18)
-#define	IEEE80211_IOCTL_WRITEREG	(SIOCIWFIRSTPRIV+19)
 #define	IEEE80211_IOCTL_DELKEY		(SIOCIWFIRSTPRIV+20)
 #define	IEEE80211_IOCTL_HALMAP		(SIOCIWFIRSTPRIV+21)
 #define	IEEE80211_IOCTL_ADDMAC		(SIOCIWFIRSTPRIV+22)
@@ -624,7 +622,6 @@ enum {
 	IEEE80211_PARAM_REGCLASS		= 59,	/* enable regclass ids in country IE */
 	IEEE80211_PARAM_DROPUNENC_EAPOL		= 60,	/* drop unencrypted eapol frames */
 	IEEE80211_PARAM_SHPREAMBLE		= 61,	/* Short Preamble */
-	IEEE80211_PARAM_DUMPREGS		= 62,	/* Pretty printed dump of Atheros hardware registers */
 	IEEE80211_PARAM_DOTH_ALGORITHM		= 63,	/* spectrum management algorithm */
 	IEEE80211_PARAM_DOTH_MINCOM   		= 64,	/* minimum number of common channels */
 	IEEE80211_PARAM_DOTH_SLCG		= 65,	/* permil of Stations Lost per Channel Gained */
