@@ -24,7 +24,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_linux.c 3902 2009-01-14 02:36:53Z proski $
+ * $Id: ieee80211_linux.c 4088 2009-08-24 22:04:45Z proski $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -178,7 +178,7 @@ EXPORT_SYMBOL(ieee80211_getmgtframe);
 /*
  * Register a vlan group.
  */
-static void
+void
 ieee80211_vlan_register(struct net_device *dev, struct vlan_group *grp)
 {
 	struct ieee80211vap *vap = netdev_priv(dev);
@@ -189,7 +189,7 @@ ieee80211_vlan_register(struct net_device *dev, struct vlan_group *grp)
 /*
  * Add an rx vlan identifier
  */
-static void
+void
 ieee80211_vlan_add_vid(struct net_device *dev, unsigned short vid)
 {
 	struct ieee80211vap *vap = netdev_priv(dev);
@@ -201,7 +201,7 @@ ieee80211_vlan_add_vid(struct net_device *dev, unsigned short vid)
 /*
  * Kill (i.e. delete) a vlan identifier.
  */
-static void
+void
 ieee80211_vlan_kill_vid(struct net_device *dev, unsigned short vid)
 {
 	struct ieee80211vap *vap = netdev_priv(dev);
@@ -219,9 +219,6 @@ ieee80211_vlan_vattach(struct ieee80211vap *vap)
 
 	dev->features |= NETIF_F_HW_VLAN_TX | NETIF_F_HW_VLAN_RX |
 			 NETIF_F_HW_VLAN_FILTER;
-	dev->vlan_rx_register = ieee80211_vlan_register;
-	dev->vlan_rx_add_vid = ieee80211_vlan_add_vid;
-	dev->vlan_rx_kill_vid = ieee80211_vlan_kill_vid;
 #endif /* IEEE80211_VLAN_TAG_USED */
 }
 
@@ -288,7 +285,7 @@ ieee80211_notify_sta_stats(struct ieee80211_node *ni)
 	static const char *tag = "STA-TRAFFIC-STAT";
 	struct net_device *dev = vap->iv_dev;
 	union iwreq_data wreq;
-	char buf[1024];
+	char buf[256];
 
 	snprintf(buf, sizeof(buf), "%s\nmac=" MAC_FMT "\nrx_packets=%u\nrx_bytes=%llu\n"
 			"tx_packets=%u\ntx_bytes=%llu\n", tag,
@@ -393,6 +390,7 @@ ieee80211_load_module(const char *modname)
 }
 
 
+#ifdef CONFIG_PROC_FS
 static struct proc_dir_entry *proc_madwifi;
 static int proc_madwifi_count = 0;
 
@@ -420,7 +418,8 @@ proc_read_nodes(struct ieee80211vap *vap, char *buf, int space)
 			p += sprintf(p, " last_rx %ld.%06ld\n",
 				     t.tv_sec, t.tv_nsec / 1000);
 			p += sprintf(p, " ni_tstamp %10llu ni_rtsf %10llu\n",
-				     le64_to_cpu(ni->ni_tstamp.tsf), ni->ni_rtsf);
+				     (unsigned long long)le64_to_cpu(ni->ni_tstamp.tsf),
+				     (unsigned long long)ni->ni_rtsf);
 		}
         }
 	IEEE80211_NODE_TABLE_UNLOCK_IRQ(nt);
@@ -428,6 +427,136 @@ proc_read_nodes(struct ieee80211vap *vap, char *buf, int space)
 	return (p - buf);
 }
 
+static int
+proc_doth_print(struct ieee80211vap *vap, char *buf, int space)
+{
+	struct ieee80211com *ic = vap->iv_ic;
+	char *p = buf;
+	struct ieee80211_channel *channel;
+	int i;
+	char str[50];
+
+	for (i = 0; i < ic->ic_nchans; i++) {
+
+		/* Assume each lines needs 500 bytes max */
+		if (buf + space < p + 500)
+			break;
+
+		channel = &ic->ic_channels[i];
+
+		if ((ic->ic_chan_non_occupy[i].tv_sec == 0) &&
+		    (ic->ic_chan_non_occupy[i].tv_usec) == 0) {
+			str[0] = 0; /* empty string */
+		} else {
+			sprintf(str, " End: %ld.%06ld",
+				ic->ic_chan_non_occupy[i].tv_sec,
+				(long)ic->ic_chan_non_occupy[i].tv_usec);
+		}
+
+		p += sprintf(p,
+			"Channel %3d (%4d MHz) : %s %s %s%s%s%s\n",
+			channel->ic_ieee,
+			channel->ic_freq,
+			isset(ic->ic_chan_active, channel->ic_ieee) ?
+			     "  Active" : "Inactive",
+			IEEE80211_IS_CHAN_PASSIVE(channel) ? "  Dfs":"NoDfs",
+			IEEE80211_IS_CHAN_RADAR(channel) ? "  Radar":"NoRadar",
+			IEEE80211_IS_CHAN_INDOOR(channel) ? " Indoor" : "",
+			IEEE80211_IS_CHAN_OUTDOOR(channel) ?  " Outdoor" : "",
+			str);
+        }
+	return (p - buf);
+}
+
+static int
+proc_doth_state_print(struct ieee80211vap *vap, char *buf, int space)
+{
+	struct ieee80211com *ic = vap->iv_ic;
+	char *p = buf;
+	struct net_device *dev = ic->ic_dev;
+	struct ath_softc *sc = netdev_priv(dev);
+
+	p += sprintf(p,
+		     "sc_curchan: --- (%4d MHz)\n",
+		     sc->sc_curchan.channel);
+
+	p += sprintf(p,
+		     "  CHANNEL_DFS:%d\n"
+		     "  CHANNEL_DFS_CLEAR:%d\n"
+		     "  CHANNEL_INTERFERENCE:%d\n"
+		     "  CAC in progress:%d\n",
+		     sc->sc_curchan.privFlags & CHANNEL_DFS ? 1 : 0,
+		     sc->sc_curchan.privFlags & CHANNEL_DFS_CLEAR ? 1 : 0,
+		     sc->sc_curchan.privFlags & CHANNEL_INTERFERENCE ? 1 : 0,
+		     timer_pending(&sc->sc_dfs_cac_timer));
+
+	p += sprintf(p,
+		     "ic_curchan: %3d (%4d MHz)\n"
+		     "  IEEE80211_F_DOTH:%d\n"
+		     "  IEEE80211_IS_CHAN_RADAR:%d\n",
+		     ic->ic_curchan->ic_ieee,
+		     ic->ic_curchan->ic_freq,
+		     ic->ic_flags & IEEE80211_F_DOTH ? 1 : 0,
+		     IEEE80211_IS_CHAN_RADAR(ic->ic_curchan) ? 1 : 0);
+
+	return (p - buf);
+}
+
+static int
+proc_iv_bss_print(struct ieee80211vap *vap, char *buf, int space)
+{
+	char *p = buf;
+	const struct ieee80211_node *ni = vap->iv_bss;
+
+	p += sprintf(p, "vap:%p vap->iv_bss: %p\n",
+		     vap, ni);
+	if (ni == NULL)
+		return (p - buf);
+
+	p += sprintf(p, "ni_macaddr:	" MAC_FMT "\n"
+		     "ni_bssid:	" MAC_FMT "\n"
+		     "ni_tstamp:	0x%llx us\n"
+		     "ni_intval:	%u TU\n"
+		     "ni_capinfo:	0x%x%s%s%s%s%s%s%s%s%s%s%s%s\n",
+		     MAC_ADDR(ni->ni_macaddr),
+		     MAC_ADDR(ni->ni_bssid),
+		     le64_to_cpu(ni->ni_tstamp.tsf),
+		     ni->ni_intval,
+		     ni->ni_capinfo,
+		     ni->ni_capinfo & IEEE80211_CAPINFO_ESS ? " ESS" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_IBSS ? " IBSS" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_CF_POLLABLE ?
+		     " CF_POLLABLE" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_CF_POLLREQ ?
+		     " CF_POLLREQ" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_PRIVACY ?
+		     " PRIVACY" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_SHORT_PREAMBLE ?
+		     " SHORT_PREAMBLE" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_PBCC ? " PBCC" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_CHNL_AGILITY ?
+		     " CHANNEL_AGILITY" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_SPECTRUM_MGMT ?
+		     " SPECTRUM_MGMT" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_SHORT_SLOTTIME ?
+		     " SHORT_SLOTTIME" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_RSN ? " RSN" : "",
+		     ni->ni_capinfo & IEEE80211_CAPINFO_DSSSOFDM ?
+		     "DSSSOFDM" : "");
+
+	if (ni->ni_chan == NULL) {
+		p += sprintf(p, "ni_chan:	NULL\n");
+	} else if (ni->ni_chan == IEEE80211_CHAN_ANYC) {
+		p += sprintf(p, "ni_chan:	ANY\n");
+	} else {
+		p += sprintf(p,
+			     "ni_chan:	Frequency:%4u MHz  Channel:%3u\n",
+			     ni->ni_chan->ic_freq, ni->ni_chan->ic_ieee);
+	}
+
+	return (p - buf);
+}
+
 static ssize_t
 proc_ieee80211_read(struct file *file, char __user *buf, size_t len, loff_t *offset)
 {
@@ -450,11 +579,9 @@ proc_ieee80211_read(struct file *file, char __user *buf, size_t len, loff_t *off
 }
 
 static int
-proc_ieee80211_open(struct inode *inode, struct file *file)
+proc_common_open(struct inode *inode, struct file *file)
 {
-	struct proc_ieee80211_priv *pv = NULL;
-	struct proc_dir_entry *dp = PDE(inode);
-	struct ieee80211vap *vap = dp->data;
+	struct proc_ieee80211_priv *pv;
 
 	if (!(file->private_data = kzalloc(sizeof(struct proc_ieee80211_priv), 
 			GFP_KERNEL)))
@@ -476,11 +603,83 @@ proc_ieee80211_open(struct inode *inode, struct file *file)
 	memset(pv->rbuf, 0, MAX_PROC_IEEE80211_SIZE);
 	pv->max_wlen = MAX_PROC_IEEE80211_SIZE;
 	pv->max_rlen = MAX_PROC_IEEE80211_SIZE;
+
+	return 0;
+}
+
+static int
+proc_ieee80211_open(struct inode *inode, struct file *file)
+{
+	struct proc_ieee80211_priv *pv = NULL;
+	struct proc_dir_entry *dp = PDE(inode);
+	struct ieee80211vap *vap = dp->data;
+	int result;
+
+	result = proc_common_open(inode, file);
+	if (result != 0)
+		return result;
+
 	/* now read the data into the buffer */
+	pv = (struct proc_ieee80211_priv *) file->private_data;
 	pv->rlen = proc_read_nodes(vap, pv->rbuf, MAX_PROC_IEEE80211_SIZE);
 	return 0;
 }
 
+static int
+proc_doth_open(struct inode *inode, struct file *file)
+{
+	struct proc_ieee80211_priv *pv = NULL;
+	struct proc_dir_entry *dp = PDE(inode);
+	struct ieee80211vap *vap = dp->data;
+	int result;
+
+	result = proc_common_open(inode, file);
+	if (result != 0)
+		return result;
+
+	/* now read the data into the buffer */
+	pv = (struct proc_ieee80211_priv *) file->private_data;
+	pv->rlen = proc_doth_print(vap, pv->rbuf, MAX_PROC_IEEE80211_SIZE);
+	return 0;
+}
+
+static int
+proc_doth_state_open(struct inode *inode, struct file *file)
+{
+	struct proc_ieee80211_priv *pv = NULL;
+	struct proc_dir_entry *dp = PDE(inode);
+	struct ieee80211vap *vap = dp->data;
+	int result;
+
+	result = proc_common_open(inode, file);
+	if (result != 0)
+		return result;
+
+	/* now read the data into the buffer */
+	pv = (struct proc_ieee80211_priv *) file->private_data;
+	pv->rlen = proc_doth_state_print(vap,
+					 pv->rbuf, MAX_PROC_IEEE80211_SIZE);
+	return 0;
+}
+
+static int
+proc_iv_bss_open(struct inode *inode, struct file *file)
+{
+	struct proc_ieee80211_priv *pv = NULL;
+	struct proc_dir_entry *dp = PDE(inode);
+	struct ieee80211vap *vap = dp->data;
+	int result;
+
+	result = proc_common_open(inode, file);
+	if (result != 0)
+		return result;
+
+	/* now read the data into the buffer */
+	pv = (struct proc_ieee80211_priv *) file->private_data;
+	pv->rlen = proc_iv_bss_print(vap, pv->rbuf, MAX_PROC_IEEE80211_SIZE);
+	return 0;
+}
+
 static ssize_t
 proc_ieee80211_write(struct file *file, const char __user *buf, size_t len, loff_t *offset)
 {
@@ -525,6 +724,28 @@ static struct file_operations proc_ieee80211_ops = {
 	.release = proc_ieee80211_close,
 };
 
+static struct file_operations proc_doth_ops = {
+	.read = proc_ieee80211_read,
+	.write = proc_ieee80211_write,
+	.open = proc_doth_open,
+	.release = proc_ieee80211_close,
+};
+
+static struct file_operations proc_doth_state_ops = {
+	.read = proc_ieee80211_read,
+	.write = proc_ieee80211_write,
+	.open = proc_doth_state_open,
+	.release = proc_ieee80211_close,
+};
+
+static struct file_operations proc_iv_bss_ops = {
+	.read = proc_ieee80211_read,
+	.write = proc_ieee80211_write,
+	.open = proc_iv_bss_open,
+	.release = proc_ieee80211_close,
+};
+#endif			/* CONFIG_PROC_FS */
+
 #ifdef IEEE80211_DEBUG
 static int
 IEEE80211_SYSCTL_DECL(ieee80211_sysctl_debug, ctl, write, filp, buffer,
@@ -724,7 +945,6 @@ ieee80211_virtfs_latevattach(struct ieee80211vap *vap)
 {
 	int i, space;
 	char *devname = NULL;
-	struct ieee80211_proc_entry *tmp = NULL;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17)
 	int ret;
 
@@ -795,6 +1015,7 @@ ieee80211_virtfs_latevattach(struct ieee80211vap *vap)
 		vap->iv_sysctls = NULL;
 	}
 
+#ifdef CONFIG_PROC_FS
 	/* Ensure the base madwifi directory exists */
 	if (!proc_madwifi && proc_net != NULL) {
 		proc_madwifi = proc_mkdir("madwifi", proc_net);
@@ -810,10 +1031,14 @@ ieee80211_virtfs_latevattach(struct ieee80211vap *vap)
 
 	/* Create a proc entry listing the associated stations */
 	ieee80211_proc_vcreate(vap, &proc_ieee80211_ops, "associated_sta");
+	ieee80211_proc_vcreate(vap, &proc_doth_ops, "doth");
+	ieee80211_proc_vcreate(vap, &proc_doth_state_ops, "doth_state");
+	ieee80211_proc_vcreate(vap, &proc_iv_bss_ops, "iv_bss");
 
 	/* Recreate any other proc entries that have been registered */
 	if (vap->iv_proc) {
-		tmp = vap->iv_proc_entries;
+		struct ieee80211_proc_entry *tmp = vap->iv_proc_entries;
+
 		while (tmp) {
 			if (!tmp->entry) {
 				tmp->entry = create_proc_entry(tmp->name,
@@ -824,8 +1049,10 @@ ieee80211_virtfs_latevattach(struct ieee80211vap *vap)
 			tmp = tmp->next;
 		}
 	}
+#endif			/* CONFIG_PROC_FS */
 }
 
+#ifdef CONFIG_PROC_FS
 /* Frees all memory used for the list of proc entries */
 void
 ieee80211_proc_cleanup(struct ieee80211vap *vap)
@@ -905,12 +1132,11 @@ ieee80211_proc_vcreate(struct ieee80211vap *vap,
 	return 0;
 }
 EXPORT_SYMBOL(ieee80211_proc_vcreate);
+#endif			/* CONFIG_PROC_FS */
 
 void
 ieee80211_virtfs_vdetach(struct ieee80211vap *vap)
 {
-	struct ieee80211_proc_entry *tmp=NULL;
-
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
 	sysfs_remove_group(&vap->iv_dev->dev.kobj, &ieee80211_attr_grp);
 #elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17)
@@ -922,9 +1148,11 @@ ieee80211_virtfs_vdetach(struct ieee80211vap *vap)
 		vap->iv_sysctl_header = NULL;
 	}
 
+#ifdef CONFIG_PROC_FS
 	if (vap->iv_proc) {
 		/* Remove child proc entries but leave them in the list */
-		tmp = vap->iv_proc_entries;
+		struct ieee80211_proc_entry *tmp = vap->iv_proc_entries;
+
 		while (tmp) {
 			if (tmp->entry) {
 				remove_proc_entry(tmp->name, vap->iv_proc);
@@ -939,6 +1167,7 @@ ieee80211_virtfs_vdetach(struct ieee80211vap *vap)
 		}
 		proc_madwifi_count--;
 	}
+#endif			/* CONFIG_PROC_FS */
 
 	if (vap->iv_sysctls && vap->iv_sysctls[2].procname) {
 		kfree(vap->iv_sysctls[2].procname);
@@ -962,8 +1191,14 @@ ieee80211_rcv_dev_event(struct notifier_block *this, unsigned long event,
 	void *ptr)
 {
 	struct net_device *dev = (struct net_device *)ptr;
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
 	if (!dev || dev->open != &ieee80211_open)
 		return 0;
+#else
+	if (!dev || dev->netdev_ops->ndo_open != &ieee80211_open)
+		return 0;
+#endif
 
 	switch (event) {
 	case NETDEV_CHANGENAME:
