@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_node.c 3941 2009-02-04 21:43:58Z proski $
+ * $Id: ieee80211_node.c 4079 2009-07-11 17:37:55Z benoit $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -75,7 +75,6 @@ static struct ieee80211_node *node_alloc(struct ieee80211vap *);
 static void node_cleanup(struct ieee80211_node *);
 static void node_free(struct ieee80211_node *);
 
-static int32_t node_count(struct ieee80211com *ic);
 static u_int8_t node_getrssi(const struct ieee80211_node *);
 
 static void node_table_leave_locked(struct ieee80211_node_table *, 
@@ -108,7 +107,6 @@ ieee80211_node_attach(struct ieee80211com *ic)
 	ic->ic_node_free = node_free;
 	ic->ic_node_cleanup = node_cleanup;
 
-	ic->ic_node_count = node_count;
 	ic->ic_node_getrssi = node_getrssi;
 }
 
@@ -564,16 +562,25 @@ ssid_equal(const struct ieee80211_node *a, const struct ieee80211_node *b)
 /*
  * Join the specified IBSS/BSS network.  The node is assumed to
  * be passed in with a reference already held for use in assigning
- * to iv_bss.
+ * to iv_bss. Returns 1 on success, 0 on failure.
  */
 static int
 ieee80211_sta_join1(struct ieee80211_node *selbs)
 {
 	struct ieee80211vap *vap = selbs->ni_vap;
 	struct ieee80211com *ic = selbs->ni_ic;
+	struct ieee80211_node *ni_bss = vap->iv_bss;
 	struct ieee80211_node *obss;
 	int canreassoc;
 
+	/* Check vap->iv_bss at the beginning */
+	if (ni_bss == NULL) {
+		IEEE80211_DPRINTF(vap, IEEE80211_MSG_ASSOC,
+				  "%s: BUG: ni_bss is NULL\n",
+				  __func__);
+		return 0;
+	}
+
 	if (vap->iv_opmode == IEEE80211_M_IBSS) {
 		/*
 		 * Delete unusable rates; we've already checked
@@ -592,11 +599,47 @@ ieee80211_sta_join1(struct ieee80211_node *selbs)
 	 */
 	canreassoc = ((obss != NULL) &&
 		(vap->iv_state == IEEE80211_S_RUN) && ssid_equal(obss, selbs));
-	vap->iv_bss = selbs;
-	IEEE80211_ADDR_COPY(vap->iv_bssid, selbs->ni_bssid);
-	if (obss != NULL)
-		ieee80211_unref_node(&obss);
-	ic->ic_bsschan = selbs->ni_chan;
+	/*
+	 * In IBSS mode, updates vap->iv_bss content from selbs. In other
+	 * modes, set vap->iv_bss = selbs
+	 */
+	if (vap->iv_opmode == IEEE80211_M_IBSS) {
+		IEEE80211_ADDR_COPY(vap->iv_bssid, selbs->ni_bssid);
+
+		IEEE80211_ADDR_COPY(ni_bss->ni_bssid, selbs->ni_bssid);
+		/* FIXME : ni_tstamp should be updated with value from beacons
+		 * only, not from probe responses since ni_tstamp is used to
+		 * synchronize beacons transmission in ath_beacon_config() */
+		ni_bss->ni_tstamp.tsf	= selbs->ni_tstamp.tsf;
+		ni_bss->ni_intval	= selbs->ni_intval;
+		ni_bss->ni_capinfo	= selbs->ni_capinfo;
+		ni_bss->ni_chan		= selbs->ni_chan;
+
+		/* Since ESSID matches, we don't need to update it */
+
+		ni_bss->ni_fhdwell	= selbs->ni_fhdwell;
+		ni_bss->ni_fhindex	= selbs->ni_fhindex;
+		ni_bss->ni_erp		= selbs->ni_erp;
+		ni_bss->ni_timoff	= selbs->ni_timoff;
+		if (selbs->ni_wme_ie != NULL)
+			ieee80211_saveie(&ni_bss->ni_wme_ie, selbs->ni_wme_ie);
+		if (selbs->ni_wpa_ie != NULL)
+			ieee80211_saveie(&ni_bss->ni_wpa_ie, selbs->ni_wpa_ie);
+		if (selbs->ni_rsn_ie != NULL)
+			ieee80211_saveie(&ni_bss->ni_rsn_ie, selbs->ni_rsn_ie);
+		if (selbs->ni_ath_ie != NULL)
+			ieee80211_saveath(ni_bss, selbs->ni_ath_ie);
+		ni_bss->ni_rates = selbs->ni_rates;
+
+		/* We got a reference to selbs, so need to unref() */
+		ieee80211_unref_node(&selbs);
+	} else {
+		vap->iv_bss = selbs;
+		IEEE80211_ADDR_COPY(vap->iv_bssid, selbs->ni_bssid);
+		if (obss != NULL)
+			ieee80211_unref_node(&obss);
+	}
+	ic->ic_bsschan = vap->iv_bss->ni_chan;
 	ic->ic_curchan = ic->ic_bsschan;
 	ic->ic_curmode = ieee80211_chan2mode(ic->ic_curchan);
 	ic->ic_set_channel(ic);
@@ -920,12 +963,6 @@ node_free(struct ieee80211_node *ni)
 	FREE(ni, M_80211_NODE);
 }
 
-static int32_t
-node_count(struct ieee80211com *ic)
-{
-	return atomic_read(&ic->ic_node_counter);
-}
-
 static u_int8_t
 node_getrssi(const struct ieee80211_node *ni)
 {
@@ -1384,7 +1421,7 @@ ieee80211_find_txnode(struct ieee80211vap *vap, const u_int8_t *mac)
 EXPORT_SYMBOL(ieee80211_find_txnode);
 
 /* Context: hwIRQ, softIRQ and process context. */
-void
+static void
 ieee80211_free_node(struct ieee80211_node *ni)
 {
 	struct ieee80211vap *vap = ni->ni_vap;
@@ -1402,7 +1439,6 @@ ieee80211_free_node(struct ieee80211_node *ni)
 
 	vap->iv_ic->ic_node_free(ni);
 }
-EXPORT_SYMBOL(ieee80211_free_node);
 
 static void _reset_node(void *arg, struct ieee80211_node *ni)
 {
@@ -1645,7 +1681,8 @@ ieee80211_dump_node(struct ieee80211_node_table *nt, struct ieee80211_node *ni)
 		       ni->ni_rxseqs[i] & IEEE80211_SEQ_FRAG_MASK);
 	}
 	printk("\trtsf %10llu rssi %u intval %u capinfo 0x%x\n",
-		ni->ni_rtsf, ni->ni_rssi, ni->ni_intval, ni->ni_capinfo);
+		(unsigned long long)ni->ni_rtsf, ni->ni_rssi, ni->ni_intval,
+		ni->ni_capinfo);
 	printk("\tbssid " MAC_FMT " essid \"%.*s\" channel %u:0x%x\n",
 		MAC_ADDR(ni->ni_bssid),
 		ni->ni_esslen, ni->ni_essid,
@@ -2102,7 +2139,7 @@ ieee80211_ref_node(struct ieee80211_node *ni)
 	if (ni == NULL) {
 		printk(KERN_ERR "%s: NULL node.\n", __func__);
 		dump_stack();
-	}else if (atomic_read(&ni->ni_refcnt) < 1) {
+	} else if (atomic_read(&ni->ni_refcnt) < 1) {
 		node_print_message(IEEE80211_MSG_ANY,
 				   0 /* show counter */, 
 				   0 /* adjust refcount */, 
@@ -2158,12 +2195,3 @@ ieee80211_unref_node(struct ieee80211_node **pni)
 	*pni = NULL;
 }
 EXPORT_SYMBOL(ieee80211_unref_node);
-
-int32_t 
-ieee80211_get_node_count(struct ieee80211com *ic)
-{
-	return node_count(ic);
-}
-EXPORT_SYMBOL(ieee80211_get_node_count);
-
-
