@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_proto.c 3734 2008-06-19 16:58:07Z proski $
+ * $Id: ieee80211_proto.c 4076 2009-07-11 17:20:58Z benoit $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -130,7 +130,6 @@ ieee80211_proto_vattach(struct ieee80211vap *vap)
 	init_timer(&vap->iv_mgtsend);
 	init_timer(&vap->iv_xrvapstart);
 	init_timer(&vap->iv_swbmiss);
-	init_timer(&vap->iv_csa_timer);
 	vap->iv_mgtsend.function = ieee80211_tx_timeout;
 	vap->iv_mgtsend.data = (unsigned long) vap;
 
@@ -952,7 +951,7 @@ ieee80211_init(struct net_device *dev, int forcescan)
 {
 #define	IS_RUNNING(_dev) \
 	((_dev->flags & (IFF_RUNNING|IFF_UP)) == (IFF_RUNNING|IFF_UP))
-	struct ieee80211vap *vap = dev->priv;
+	struct ieee80211vap *vap = netdev_priv(dev);
 	struct ieee80211com *ic = vap->iv_ic;
 	struct net_device *parent = ic->ic_dev;
 
@@ -1095,7 +1094,7 @@ EXPORT_SYMBOL(ieee80211_start_running);
 int
 ieee80211_stop(struct net_device *dev)
 {
-	struct ieee80211vap *vap = dev->priv;
+	struct ieee80211vap *vap = netdev_priv(dev);
 	struct ieee80211com *ic = vap->iv_ic;
 	struct net_device *parent = ic->ic_dev;
 
@@ -1405,7 +1404,13 @@ __ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int
 			if (arg != 0)
 				ieee80211_scan_assoc_fail(ic,
 					vap->iv_bss->ni_macaddr, arg);
-			if (ic->ic_roaming == IEEE80211_ROAMING_AUTO)
+
+			/* ic_roaming is relevant to STA mode only. Since DFS
+			 * CAC does a SCAN -> SCAN transition, this code was
+			 * causing a spurious scan that was stopping DFS CAC
+			 * altogether */
+			if (vap->iv_opmode == IEEE80211_M_STA &&
+			    ic->ic_roaming == IEEE80211_ROAMING_AUTO)
 				ieee80211_check_scan(vap,
 					IEEE80211_SCAN_ACTIVE,
 					IEEE80211_SCAN_FOREVER,
@@ -1466,7 +1471,8 @@ __ieee80211_newstate(struct ieee80211vap *vap, enum ieee80211_state nstate, int
 				break;
 			case IEEE80211_FC0_SUBTYPE_DEAUTH:
 				ieee80211_sta_leave(ni);
-				if (ic->ic_roaming == IEEE80211_ROAMING_AUTO) {
+				if (vap->iv_opmode == IEEE80211_M_STA &&
+				    ic->ic_roaming == IEEE80211_ROAMING_AUTO) {
 					/* try to reauth */
 					IEEE80211_SEND_MGMT(ni,
 						IEEE80211_FC0_SUBTYPE_AUTH, 1);
