@@ -1,5 +1,6 @@
 /*-
  * Copyright (c) 2002-2005 Sam Leffler, Errno Consulting
+ * Copyright (c) 2005  Matt Mackall <mpm@selenic.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_scan_ap.c 2988 2007-11-28 05:36:29Z proski $
+ * $Id: ieee80211_scan_ap.c 4076 2009-07-11 17:20:58Z benoit $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -47,18 +48,94 @@
 #include <linux/init.h>
 #include <linux/delay.h>
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,11)
+/* Copied from Linux lib/sort.c */
+static void u32_swap(void *a, void *b, int size)
+{
+	u32 t = *(u32 *)a;
+	*(u32 *)a = *(u32 *)b;
+	*(u32 *)b = t;
+}
+
+static void generic_swap(void *a, void *b, int size)
+{
+	char t;
+
+	do {
+		t = *(char *)a;
+		*(char *)a++ = *(char *)b;
+		*(char *)b++ = t;
+	} while (--size > 0);
+}
+
+/**
+ * sort - sort an array of elements
+ * @base: pointer to data to sort
+ * @num: number of elements
+ * @size: size of each element
+ * @cmp: pointer to comparison function
+ * @swap: pointer to swap function or NULL
+ *
+ * This function does a heapsort on the given array. You may provide a
+ * swap function optimized to your element type.
+ *
+ * Sorting time is O(n log n) both on average and worst-case. While
+ * qsort is about 20% faster on average, it suffers from exploitable
+ * O(n*n) worst-case behavior and extra memory requirements that make
+ * it less suitable for kernel use.
+ */
+
+void sort(void *base, size_t num, size_t size,
+	  int (*cmp)(const void *, const void *),
+	  void (*swap)(void *, void *, int size))
+{
+	/* pre-scale counters for performance */
+	int i = (num/2 - 1) * size, n = num * size, c, r;
+
+	if (!swap)
+		swap = (size == 4 ? u32_swap : generic_swap);
+
+	/* heapify */
+	for ( ; i >= 0; i -= size) {
+		for (r = i; r * 2 + size < n; r  = c) {
+			c = r * 2 + size;
+			if (c < n - size && cmp(base + c, base + c + size) < 0)
+				c += size;
+			if (cmp(base + r, base + c) >= 0)
+				break;
+			swap(base + r, base + c, size);
+		}
+	}
+
+	/* sort */
+	for (i = n - size; i > 0; i -= size) {
+		swap(base, base + i, size);
+		for (r = 0; r * 2 + size < i; r = c) {
+			c = r * 2 + size;
+			if (c < i - size && cmp(base + c, base + c + size) < 0)
+				c += size;
+			if (cmp(base + r, base + c) >= 0)
+				break;
+			swap(base + r, base + c, size);
+		}
+	}
+}
+#else
+#include <linux/sort.h>
+#endif
+
 #include "if_media.h"
 
 #include <net80211/ieee80211_var.h>
 
-#define	AP_PURGE_SCANS	2		/* age for purging entries (scans) */
+#define	AP_PURGE_SCANS	2			/* age for purging entries (scans) */
 #define RSSI_LPF_LEN	10
-#define	RSSI_EP_MULTIPLIER	(1<<7)	/* pow2 to optimize out * and / */
+#define	RSSI_EP_MULTIPLIER	(1 << 7)	/* pow2 to optimize out * and / */
 #define RSSI_IN(x)		((x) * RSSI_EP_MULTIPLIER)
 #define LPF_RSSI(x, y, len)	(((x) * ((len) - 1) + (y)) / (len))
 #define RSSI_LPF(x, y) do {						\
 	if ((y) >= -20)							\
-		x = LPF_RSSI((x), RSSI_IN((y)), RSSI_LPF_LEN);			\
+		x = LPF_RSSI((x), RSSI_IN((y)), RSSI_LPF_LEN);		\
 } while (0)
 #define	EP_RND(x, mul) \
 	((((x)%(mul)) >= ((mul)/2)) ? howmany(x, mul) : (x)/(mul))
@@ -89,8 +166,8 @@ struct scan_entry {
 	struct ieee80211_scan_entry base;
 	TAILQ_ENTRY(scan_entry) se_list;
 	LIST_ENTRY(scan_entry) se_hash;
-	u_int8_t	se_seen;		/* seen during current scan */
-	u_int8_t	se_notseen;		/* not seen in previous scans */
+	u_int8_t	se_seen;	/* seen during current scan */
+	u_int8_t	se_notseen;	/* not seen in previous scans */
 	u_int32_t se_avgrssi;		/* LPF rssi state */
 	unsigned long se_lastupdate;	/* time of last update */
 	unsigned long se_lastfail;	/* time of last failure */
@@ -99,14 +176,18 @@ struct scan_entry {
 };
 
 struct ap_state {
-	unsigned int as_vap_desired_mode;       /* Used for channel selection, vap->iv_des_mode */
-	unsigned int as_required_mode;          /* Used for channel selection, filtered version of as_vap_desired_mode */
+	unsigned int as_vap_desired_mode;       /* Used for channel selection, 
+						 * vap->iv_des_mode */
+	unsigned int as_required_mode;          /* Used for channel selection, 
+						 * filtered version of 
+						 * as_vap_desired_mode */
 	int as_maxrssi[IEEE80211_CHAN_MAX]; 	/* Used for channel selection */
 
 	/* These fields are just for scan caching for returning responses to
 	 * wireless extensions.  i.e. show peers, APs, etc. */
 	spinlock_t as_lock;			/* on scan table */
-	int as_newscan;				/* trigger for updating seen/not-seen for aging */
+	int as_newscan;				/* trigger for updating 
+						 * seen/not-seen for aging */
 	TAILQ_HEAD(, scan_entry) as_entry;	/* all entries */
 	ATH_LIST_HEAD(, scan_entry) as_hash[AP_HASHSIZE];
 	spinlock_t as_scanlock;			/* on as_scangen */
@@ -119,7 +200,8 @@ struct ap_state {
 
 static int ap_flush(struct ieee80211_scan_state *);
 static void action_tasklet(IEEE80211_TQUEUE_ARG);
-static struct ieee80211_channel *find11gchannel(struct ieee80211com *ic, int i, int freq);
+static struct ieee80211_channel *find11gchannel(struct ieee80211com *ic, 
+		int i, int freq);
 
 static const u_int chanflags[] = {
 	IEEE80211_CHAN_B,	/* IEEE80211_MODE_AUTO */
@@ -127,12 +209,16 @@ static const u_int chanflags[] = {
 	IEEE80211_CHAN_B,	/* IEEE80211_MODE_11B */
 	IEEE80211_CHAN_PUREG,	/* IEEE80211_MODE_11G */
 	IEEE80211_CHAN_FHSS,	/* IEEE80211_MODE_FH */
-	IEEE80211_CHAN_A,	/* IEEE80211_MODE_TURBO_A */ /* for turbo mode look for AP in normal channel */
+	IEEE80211_CHAN_A,	/* IEEE80211_MODE_TURBO_A */ /* for turbo mode 
+							      * look for AP in 
+							      * normal channel 
+							      */
 	IEEE80211_CHAN_PUREG,	/* IEEE80211_MODE_TURBO_G */
 	IEEE80211_CHAN_ST,	/* IEEE80211_MODE_TURBO_STATIC_A */
 };
 
-static const u_int16_t rcl1[] =		/* 8 FCC channel: 52, 56, 60, 64, 36, 40, 44, 48 */
+static const u_int16_t rcl1[] =		/* 8 FCC channel: 52, 56, 60, 64, 
+					 *                36, 40, 44, 48 */
 { 5260, 5280, 5300, 5320, 5180, 5200, 5220, 5240 };
 static const u_int16_t rcl2[] =		/* 4 MKK channels: 34, 38, 42, 46 */
 { 5170, 5190, 5210, 5230 };
@@ -140,7 +226,9 @@ static const u_int16_t rcl3[] =		/* 2.4Ghz ch: 1,6,11,7,13 */
 { 2412, 2437, 2462, 2442, 2472 };
 static const u_int16_t rcl4[] =		/* 5 FCC channel: 149, 153, 161, 165 */
 { 5745, 5765, 5785, 5805, 5825 };
-static const u_int16_t rcl7[] =		/* 11 ETSI channel: 100,104,108,112,116,120,124,128,132,136,140 */
+static const u_int16_t rcl7[] =		/* 11 ETSI channel: 100, 104, 108, 112,
+					 *                  116, 120, 124, 128, 
+					 *                  132, 136, 140 */
 { 5500, 5520, 5540, 5560, 5580, 5600, 5620, 5640, 5660, 5680, 5700 };
 static const u_int16_t rcl8[] =		/* 2.4Ghz ch: 2,3,4,5,8,9,10,12 */
 { 2417, 2422, 2427, 2432, 2447, 2452, 2457, 2467 };
@@ -170,22 +258,22 @@ struct scanlist {
 };
 
 #define	IEEE80211_MODE_TURBO_STATIC_A	IEEE80211_MODE_MAX
-#define	X(a)	.count = sizeof(a)/sizeof(a[0]), .list = a
+#define	X(a)	.count = ARRAY_SIZE(a), .list = a
 
 static const struct scanlist staScanTable[] = {
-	{ IEEE80211_MODE_11B,   		X(rcl3) },
-	{ IEEE80211_MODE_11A,   		X(rcl1) },
-	{ IEEE80211_MODE_11A,   		X(rcl2) },
-	{ IEEE80211_MODE_11B,   		X(rcl8) },
-	{ IEEE80211_MODE_11B,   		X(rcl9) },
-	{ IEEE80211_MODE_11A,   		X(rcl4) },
+	{ IEEE80211_MODE_11B,   		X(rcl3)  },
+	{ IEEE80211_MODE_11A,   		X(rcl1)  },
+	{ IEEE80211_MODE_11A,   		X(rcl2)  },
+	{ IEEE80211_MODE_11B,   		X(rcl8)  },
+	{ IEEE80211_MODE_11B,   		X(rcl9)  },
+	{ IEEE80211_MODE_11A,   		X(rcl4)  },
 #ifdef ATH_TURBO_SCAN
-	{ IEEE80211_MODE_TURBO_STATIC_A,	X(rcl5) },
-	{ IEEE80211_MODE_TURBO_STATIC_A,	X(rcl6) },
+	{ IEEE80211_MODE_TURBO_STATIC_A,	X(rcl5)  },
+	{ IEEE80211_MODE_TURBO_STATIC_A,	X(rcl6)  },
 	{ IEEE80211_MODE_TURBO_A,		X(rcl6x) },
 	{ IEEE80211_MODE_TURBO_A,		X(rcl13) },
 #endif /* ATH_TURBO_SCAN */
-	{ IEEE80211_MODE_11A,			X(rcl7) },
+	{ IEEE80211_MODE_11A,			X(rcl7)  },
 	{ IEEE80211_MODE_11B,			X(rcl10) },
 	{ IEEE80211_MODE_11A,			X(rcl11) },
 #ifdef ATH_TURBO_SCAN
@@ -209,13 +297,11 @@ add_channels(struct ieee80211com *ic,
 	modeflags = chanflags[mode];
 	for (i = 0; i < nfreq; i++) {
 		c = ieee80211_find_channel(ic, freq[i], modeflags);
-		if (c == NULL || isclr(ic->ic_chan_active, c->ic_ieee))
+		if ((c == NULL) || isclr(ic->ic_chan_active, c->ic_ieee))
 			continue;
 		if (mode == IEEE80211_MODE_AUTO) {
-			/*
-			 * XXX special-case 11b/g channels so we select
-			 *     the g channel if both are present.
-			 */
+			/* XXX special-case 11b/g channels so we select
+			 *     the g channel if both are present. */
 			if (IEEE80211_IS_CHAN_B(c) &&
 			    (cg = find11gchannel(ic, i, c->ic_freq)) != NULL)
 				c = cg;
@@ -319,19 +405,17 @@ find11gchannel(struct ieee80211com *ic, int i, int freq)
 	struct ieee80211_channel *c;
 	int j;
 
-	/*
-	 * The normal ordering in the channel list is b channel
+	/* The normal ordering in the channel list is b channel
 	 * immediately followed by g so optimize the search for
-	 * this.  We'll still do a full search just in case.
-	 */
-	for (j = i+1; j < ic->ic_nchans; j++) {
+	 * this.  We'll still do a full search just in case. */
+	for (j = i + 1; j < ic->ic_nchans; j++) {
 		c = &ic->ic_channels[j];
-		if (c->ic_freq == freq && IEEE80211_IS_CHAN_ANYG(c))
+		if ((c->ic_freq == freq) && IEEE80211_IS_CHAN_ANYG(c))
 			return c;
 	}
 	for (j = 0; j < i; j++) {
 		c = &ic->ic_channels[j];
-		if (c->ic_freq == freq && IEEE80211_IS_CHAN_ANYG(c))
+		if ((c->ic_freq == freq) && IEEE80211_IS_CHAN_ANYG(c))
 			return c;
 	}
 	return NULL;
@@ -357,8 +441,8 @@ ap_start(struct ieee80211_scan_state *ss, struct ieee80211vap *vap)
 	as->as_required_mode    = 0;
 	if (as->as_vap_desired_mode != IEEE80211_MODE_AUTO) {
 		as->as_required_mode = chanflags[as->as_vap_desired_mode];
-		if (vap->iv_ath_cap & IEEE80211_ATHC_TURBOP && 
-		    as->as_required_mode != IEEE80211_CHAN_ST) {
+		if ((vap->iv_ath_cap & IEEE80211_ATHC_TURBOP) && 
+		    (as->as_required_mode != IEEE80211_CHAN_ST)) {
 			/* Fixup for dynamic turbo flags */
 			if (as->as_vap_desired_mode == IEEE80211_MODE_11G)
 				as->as_required_mode = IEEE80211_CHAN_108G;
@@ -373,30 +457,32 @@ ap_start(struct ieee80211_scan_state *ss, struct ieee80211vap *vap)
 	 * list not in the master list will be discarded. */
 	for (sl = staScanTable; sl->list != NULL; sl++) {
 		mode = sl->mode;
+
 		/* The scan table marks 2.4Ghz channels as b
 		 * so if the desired mode is 11g, then use
 		 * the 11b channel list but upgrade the mode. */
 		if (as->as_vap_desired_mode &&
-		    as->as_vap_desired_mode != mode && 
-		    as->as_vap_desired_mode == IEEE80211_MODE_11G && 
-		    mode == IEEE80211_MODE_11B)
+		    (as->as_vap_desired_mode != mode) && 
+		    (as->as_vap_desired_mode == IEEE80211_MODE_11G) && 
+		    (mode == IEEE80211_MODE_11B))
 			mode = IEEE80211_MODE_11G;
+
 		/* If we are in "AUTO" mode, upgrade the mode to auto. 
 		 * This lets add_channels upgrade an 11b channel to 
 		 * 11g if available. */
-		if (!as->as_vap_desired_mode && mode == IEEE80211_MODE_11B)
+		if (!as->as_vap_desired_mode && (mode == IEEE80211_MODE_11B))
 			mode = IEEE80211_MODE_AUTO;
+
 		/* Add the list of the channels; any that are not
 		 * in the master channel list will be discarded. */
 		add_channels(ic, ss, mode, sl->list, sl->count);
 	}
 
-	/*
-	 * Add the channels from the ic (from HAL) that are not present
-	 * in the staScanTable, assuming they pass the sanity checks...
-	 */
+	/* Add the channels from the ic (from HAL) that are not present
+	 * in the staScanTable, assuming they pass the sanity checks... */
 	for (i = 0; i < ic->ic_nchans; i++) {
 		c = &ic->ic_channels[i];
+
 		/* XR is not supported on turbo channels */
 		if (IEEE80211_IS_CHAN_TURBO(c) && vap->iv_flags & IEEE80211_F_XR)
 			continue;
@@ -418,7 +504,7 @@ ap_start(struct ieee80211_scan_state *ss, struct ieee80211vap *vap)
 			continue;
 
 		/* Make sure the channel is active */
-		if (c == NULL || isclr(ic->ic_chan_active, c->ic_ieee))
+		if ((c == NULL) || isclr(ic->ic_chan_active, c->ic_ieee))
 			continue;
 
 		/* Don't overrun */
@@ -434,9 +520,9 @@ ap_start(struct ieee80211_scan_state *ss, struct ieee80211vap *vap)
 
 #ifdef IEEE80211_DEBUG
 	if (ieee80211_msg_scan(vap)) {
-		printf("%s: scan set ", vap->iv_dev->name);
+		printk("%s: scan set ", vap->iv_dev->name);
 		ieee80211_scan_dump_channels(ss);
-		printf(" dwell min %ld max %ld\n",
+		printk(" dwell min %ld max %ld\n",
 			ss->ss_mindwell, ss->ss_maxdwell);
 	}
 #endif /* IEEE80211_DEBUG */
@@ -477,34 +563,35 @@ ap_add(struct ieee80211_scan_state *ss, const struct ieee80211_scanparams *sp,
 {
 	struct ap_state *as              = ss->ss_priv;
 	const u_int8_t *macaddr          = wh->i_addr2;
-	struct ieee80211vap *vap         = NULL;
-	struct ieee80211com *ic          = NULL;
+	struct ieee80211vap *vap         = ss->ss_vap;
+	struct ieee80211com *ic          = vap->iv_ic;
 	struct scan_entry *se            = NULL;
 	struct ieee80211_scan_entry *ise = NULL;
-	int hash;
-	int chan = 0;
+	int hash = AP_HASH(macaddr);
+	int chan;
 
 	/* This section provides scan results to wireless extensions */
 	SCAN_AP_LOCK_IRQ(as);
+
+	chan = ieee80211_chan2ieee(ic, ic->ic_curchan);
 	/* This is the only information used for channel selection by AP */
 	if (rssi > as->as_maxrssi[chan])
 		as->as_maxrssi[chan] = rssi;
-	vap = ss->ss_vap;
-	ic = vap->iv_ic;
-	hash = AP_HASH(macaddr);
-	chan = ieee80211_chan2ieee(ic, ic->ic_curchan);
 	LIST_FOREACH(se, &as->as_hash[hash], se_hash)
 		if (IEEE80211_ADDR_EQ(se->base.se_macaddr, macaddr) &&
-		    sp->ssid[1] == se->base.se_ssid[1] &&
-		    !memcmp(se->base.se_ssid+2, sp->ssid+2, se->base.se_ssid[1]))
+		    (sp->ssid[1] == se->base.se_ssid[1]) &&
+		    !memcmp(se->base.se_ssid + 2, sp->ssid + 2, 
+			    se->base.se_ssid[1]))
 			goto found;
 
 	MALLOC(se, struct scan_entry *, sizeof(struct scan_entry),
 		M_80211_SCAN, M_NOWAIT | M_ZERO);
+
 	if (se == NULL) {
 		SCAN_AP_UNLOCK_IRQ_EARLY(as);
 		return 0;
 	}
+
 	se->se_scangen = as->as_scangen-1;
 	IEEE80211_ADDR_COPY(se->base.se_macaddr, macaddr);
 	TAILQ_INSERT_TAIL(&as->as_entry, se, se_list);
@@ -530,11 +617,12 @@ found:
 	IEEE80211_ADDR_COPY(ise->se_bssid, wh->i_addr3);
 
 	/* Record RSSI data using extended precision LPF filter.*/
-	if (se->se_lastupdate == 0)			/* First sample */
+	if (se->se_lastupdate == 0)		/* First sample */
 		se->se_avgrssi = RSSI_IN(rssi);
 	else					/* Avg. w/ previous samples */
 		RSSI_LPF(se->se_avgrssi, rssi);
 	se->base.se_rssi = RSSI_GET(se->se_avgrssi);
+
 	ise->se_rtsf = rtsf;
 	memcpy(ise->se_tstamp.data, sp->tstamp, sizeof(ise->se_tstamp));
 	ise->se_intval = sp->bintval;
@@ -544,11 +632,13 @@ found:
 	ise->se_fhindex = sp->fhindex;
 	ise->se_erp = sp->erp;
 	ise->se_timoff = sp->timoff;
+
 	if (sp->tim != NULL) {
 		const struct ieee80211_tim_ie *tim =
-		    (const struct ieee80211_tim_ie *) sp->tim;
+		    (const struct ieee80211_tim_ie *)sp->tim;
 		ise->se_dtimperiod = tim->tim_period;
 	}
+
 	saveie(&ise->se_wme_ie, sp->wme);
 	saveie(&ise->se_wpa_ie, sp->wpa);
 	saveie(&ise->se_rsn_ie, sp->rsn);
@@ -563,19 +653,261 @@ found:
 	return 1;
 }
 
-/*
- * Pick a quiet channel to use for ap operation.
- */
+struct pc_params {
+	struct ieee80211vap *vap;
+	struct ieee80211_scan_state *ss;
+	int flags;
+};
+
+struct channel {
+	struct ieee80211_channel *chan;
+	int orig;
+	struct pc_params *params;
+};
+
+/* This function must be invoked with locks acquired */
+static int
+pc_cmp_radar(struct ieee80211_channel *a, struct ieee80211_channel *b)
+{
+	/* a is better than b (return < 0) when b is marked while a is not */
+	return !!IEEE80211_IS_CHAN_RADAR(a) - !!IEEE80211_IS_CHAN_RADAR(b);
+}
+
+/* This function must be invoked with locks acquired */
+static int
+pc_cmp_keepmode(struct pc_params *params, struct ieee80211_channel *a,
+		struct ieee80211_channel *b)
+{
+	struct ieee80211com *ic = params->vap->iv_ic;
+	struct ieee80211_channel *cur = ic->ic_bsschan;
+
+	if (!(params->flags & IEEE80211_SCAN_KEEPMODE))
+		return 0;
+
+	/* a is better than b (return < 0) when (a, cur) have the same mode 
+	 * and (b, cur) do not. */
+	return
+		!!IEEE80211_ARE_CHANS_SAME_MODE(b, cur) -
+		!!IEEE80211_ARE_CHANS_SAME_MODE(a, cur);
+}
+
+/* This function must be invoked with locks acquired */
+static int
+pc_cmp_sc(struct ieee80211com *ic, struct ieee80211_channel *a,
+		struct ieee80211_channel *b)
+{
+	/* a is better than b (return < 0) when a has more chan nodes than b */
+	return
+		ic->ic_chan_nodes[b->ic_ieee] -
+		ic->ic_chan_nodes[a->ic_ieee];
+}
+
+/* This function must be invoked with locks acquired */
+static int
+pc_cmp_rssi(struct ap_state *as, struct ieee80211_channel *a,
+		struct ieee80211_channel *b)
+{
+	/* a is better than b (return < 0) when a has rssi less than b */
+	return
+		as->as_maxrssi[a->ic_ieee] -
+		as->as_maxrssi[b->ic_ieee];
+}
+
+/* This function must be invoked with locks acquired */
+static int
+pc_cmp_samechan(struct ieee80211com *ic, struct ieee80211_channel *a,
+		struct ieee80211_channel *b)
+{
+	struct ieee80211_channel *ic_bsschan = ic->ic_bsschan;
+	if (ic_bsschan == IEEE80211_CHAN_ANYC)
+		return 0;
+	/* a is better than b (return < 0) when a is current (and b is not) */
+	return (b == ic_bsschan) - (a == ic_bsschan);
+}
+
+/* This function must be invoked with locks acquired */
+static int
+pc_cmp_orig(struct channel *a, struct channel *b)
+{
+	return a->orig - b->orig;
+}
+
+/* This function must be invoked with locks acquired */
+static int
+pc_cmp(const void *_a, const void *_b)
+{
+	struct ieee80211_channel *a = ((struct channel *)_a)->chan;
+	struct ieee80211_channel *b = ((struct channel *)_b)->chan;
+	struct pc_params *params = ((struct channel *)_a)->params;
+	struct ieee80211com *ic = params->vap->iv_ic;
+	int res;
+
+#define EVALUATE_CRITERION(name, ...) do {			\
+	if ((res = pc_cmp_##name(__VA_ARGS__)) != 0) 		\
+		return res;					\
+} while (0)
+
+	EVALUATE_CRITERION(radar, a, b);
+	EVALUATE_CRITERION(keepmode, params, a, b);
+	EVALUATE_CRITERION(sc, ic, a, b);
+	/* XXX: rssi useless? pick_channel evaluates it anyway */
+	EVALUATE_CRITERION(rssi, params->ss->ss_priv, a, b);
+	EVALUATE_CRITERION(samechan, ic, a, b);
+	EVALUATE_CRITERION(orig, (struct channel *)_a, (struct channel *)_b);
+
+#undef EVALUATE_CRITERION
+	return res;
+}
+
+/* This function must be invoked with locks acquired */
+static void
+pc_swap(void *a, void *b, int n)
+{
+	struct ieee80211_channel *t = ((struct channel *)a)->chan;
+	int i;
+
+	((struct channel *)a)->chan = ((struct channel *)b)->chan;
+	((struct channel *)b)->chan = t;
+
+	i = ((struct channel *)a)->orig;
+	((struct channel *)a)->orig = ((struct channel *)b)->orig;
+	((struct channel *)b)->orig = i;
+
+	/* (struct channel *)x->params doesn't have to be swapped, because it 
+	 * is the same across all channels */
+}
+
+/* Pick a quiet channel to use for ap operation.
+ * Must be invoked while we hold the locks. */
+static struct ieee80211_channel *
+pick_channel(struct ieee80211_scan_state *ss, struct ieee80211vap *vap, 
+		u_int32_t flags)
+{
+	struct ieee80211com *ic = vap->iv_ic;
+	unsigned int i, best_rssi;
+	int ss_last = ss->ss_last;
+	struct ieee80211_channel *best;
+	struct ap_state *as = ss->ss_priv;
+	struct channel chans[ss_last]; /* actually ss_last-1 is required */
+	struct channel *c = NULL;
+	struct pc_params params = { vap, ss, flags };
+	int benefit = 0;
+	int sta_assoc = 0;
+
+	for (i = 0; i < ss_last; i++) {
+		chans[i].chan = ss->ss_chans[i];
+		chans[i].orig = i;
+		chans[i].params = &params;
+	}
+
+	sort(chans, ss_last, sizeof(*chans), pc_cmp, pc_swap);
+
+#ifdef IEEE80211_DEBUG
+	for (i = 0; i < ss_last; i++) {
+		int chan = ieee80211_chan2ieee(ic, chans[i].chan);
+		IEEE80211_DPRINTF(vap, IEEE80211_MSG_SCAN, "%s: channel %u, "
+				"rssi %d, radar %d, cn %d, km %d\n",
+				__func__, chan, as->as_maxrssi[chan], 
+				IEEE80211_IS_CHAN_RADAR(chans[i].chan),
+				ic->ic_chan_nodes[chans[i].chan->ic_ieee],
+				!!IEEE80211_ARE_CHANS_SAME_MODE(chans[i].chan, 
+					ic->ic_bsschan));
+	}
+#endif /* IEEE80211_DEBUG */
+
+	best = NULL;
+	best_rssi = 0xff; /* If signal is bigger than 0xff, we'd be melting. */
+
+	for (i = 0; i < ss_last; i++) {
+		c = &chans[i];
+		benefit = best_rssi - as->as_maxrssi[c->chan->ic_ieee];
+		sta_assoc = ic->ic_sta_assoc;
+
+		/* Don't switch... */
+		if (benefit <= 0)
+			continue;
+
+		/* Verify channel is not marked for non-occupancy */
+		if (IEEE80211_IS_CHAN_RADAR(c->chan) &&
+		    (ic->ic_flags & IEEE80211_F_DOTH))
+			continue;
+
+		/* Do not select 802.11a ST if mode is specified and is not 
+		 * 802.11a ST */
+		if (as->as_required_mode &&
+		    IEEE80211_IS_CHAN_STURBO(c->chan) &&
+		    (as->as_vap_desired_mode != IEEE80211_MODE_TURBO_STATIC_A))
+			continue;
+
+		/* Verify mode matches any fixed mode specified */
+		if ((c->chan->ic_flags & as->as_required_mode) != 
+				as->as_required_mode)
+			continue;
+
+		if ((ic->ic_bsschan != NULL) &&
+			(ic->ic_bsschan != IEEE80211_CHAN_ANYC)) {
+
+			/* Make sure the channels are the same mode */
+			if ((flags & IEEE80211_SCAN_KEEPMODE) &&
+				!IEEE80211_ARE_CHANS_SAME_MODE(c->chan,
+					ic->ic_bsschan))
+				/* break the loop as the subsequent chans won't be 
+				 * better */
+				break;
+
+			if (!IEEE80211_ARE_CHANS_SAME_MODE(c->chan,
+				ic->ic_bsschan))
+				/* break the loop as the subsequent chans won't be
+				 * better */
+				break;
+		}
+
+		if (sta_assoc != 0) {
+			int sl = ic->ic_cn_total - 
+				ic->ic_chan_nodes[c->chan->ic_ieee]; /* count */
+			if (ic->ic_sc_algorithm == IEEE80211_SC_LOOSE) {
+				int sl_max = ic->ic_sc_sldg * benefit;
+				sl = 1000 * sl / sta_assoc; /* permil */
+				IEEE80211_DPRINTF(vap, IEEE80211_MSG_SCAN,
+						"%s: chan %d, dB gained: %d, "
+						"STAs lost: %d permil (max %d)\n",
+						__func__, c->chan->ic_ieee, 
+						benefit, sl, sl_max);
+				if (sl > sl_max)
+					continue;
+			} else if (((ic->ic_sc_algorithm == 
+						 IEEE80211_SC_TIGHT) ||
+					(ic->ic_sc_algorithm == 
+						 IEEE80211_SC_STRICT)) && 
+					(sl > 0)) {
+				/* Break the loop as the subsequent chans 
+				 * won't be better. */
+				break;
+			}
+		}
+		best = c->chan;
+		best_rssi = as->as_maxrssi[best->ic_ieee];
+	}
+
+	if (best != NULL) {
+		i = best->ic_ieee;
+		IEEE80211_DPRINTF(vap, IEEE80211_MSG_SCAN,
+				"%s: best: channel %u rssi %d\n",
+				__func__, i, as->as_maxrssi[i]);
+	}
+	return best;
+}
+
 static int
 ap_end(struct ieee80211_scan_state *ss, struct ieee80211vap *vap,
 		int (*action)(struct ieee80211vap *, 
 			const struct ieee80211_scan_entry *), 
-			u_int32_t flags)
+		u_int32_t flags)
 {
 	struct ap_state *as = ss->ss_priv;
+	struct ieee80211_channel *bestchan = NULL;
 	struct ieee80211com *ic = NULL;
 	int res = 1;
-	int i, chan, bestchan, bestchanix;
 
 	SCAN_AP_LOCK_IRQ(as);
 
@@ -583,87 +915,41 @@ ap_end(struct ieee80211_scan_state *ss, struct ieee80211vap *vap,
 		("wrong opmode %u", vap->iv_opmode));
 
 	ic = vap->iv_ic;
-	/* XXX select channel more intelligently, e.g. channel spread, power */
-	bestchan = -1;
-	bestchanix = 0;		/* NB: silence compiler */
-	/* NB: use scan list order to preserve channel preference */
-	for (i = 0; i < ss->ss_last; i++) {
-		/*
-		 * If the channel is unoccupied the max rssi
-		 * should be zero; just take it.  Otherwise
-		 * track the channel with the lowest rssi and
-		 * use that when all channels appear occupied.
-		 */
-		/*
-		 * Check for channel interference, and if found,
-		 * skip the channel.  We assume that all channels
-		 * will be checked so atleast one can be found
-		 * suitable and will change.  IF this changes,
-		 * then we must know when we "have to" change
-		 * channels for radar and move off.
-		 */
-
-		if (ss->ss_chans[i]->ic_flags & IEEE80211_CHAN_RADAR)
-			continue;
-		if (flags & IEEE80211_SCAN_KEEPMODE) {
-			if (ic->ic_curchan != NULL) {
-				if ((ss->ss_chans[i]->ic_flags & IEEE80211_CHAN_ALLTURBO) != (ic->ic_curchan->ic_flags & IEEE80211_CHAN_ALLTURBO))
-					continue;
-			}
-		}
-
-		chan = ieee80211_chan2ieee(ic, ss->ss_chans[i]);
-
-		IEEE80211_DPRINTF(vap, IEEE80211_MSG_SCAN,
-			"%s: channel %u rssi %d bestchan %d bestchan rssi %d\n",
-			__func__, chan, as->as_maxrssi[chan],
-			bestchan, bestchan != -1 ? as->as_maxrssi[bestchan] : 0);
-
-		if (as->as_maxrssi[chan] == 0) {
-			bestchan = chan;
-			bestchanix = i;
-			/* XXX use other considerations */
-			break;
-		}
-		if (bestchan == -1 ||
-		    as->as_maxrssi[chan] < as->as_maxrssi[bestchan])
-			bestchan = chan;
-	}
-	if (bestchan == -1) {
+	bestchan = pick_channel(ss, vap, flags);
+	if (bestchan == NULL) {
 		if (ss->ss_last > 0) {
 			/* no suitable channel, should not happen */
-			IEEE80211_DPRINTF(vap, IEEE80211_MSG_SCAN,
-				"%s: no suitable channel! (should not happen)\n", __func__);
-			/* XXX print something? */
+			printk(KERN_ERR "%s: %s: no suitable channel! "
+				"(should not happen)\n", 
+				DEV_NAME(vap->iv_dev), __func__);
 		}
-		res = 0; /* restart scan */
+		res = 1; /* Do NOT restart scan */
 	} else {
-		struct ieee80211_channel *c;
 		struct ieee80211_scan_entry se;
 		/* XXX: notify all VAPs? */
-		/* if this is a dynamic turbo frequency , start with normal mode first */
-
-		c = ss->ss_chans[bestchanix];
-		if (IEEE80211_IS_CHAN_TURBO(c) && !IEEE80211_IS_CHAN_STURBO(c)) {
-			if ((c = ieee80211_find_channel(ic, c->ic_freq,
-				c->ic_flags & ~IEEE80211_CHAN_TURBO)) == NULL) {
+		/* if this is a dynamic turbo frequency , start with normal 
+		 * mode first */
+		if (IEEE80211_IS_CHAN_TURBO(bestchan) && 
+				!IEEE80211_IS_CHAN_STURBO(bestchan)) {
+			if ((bestchan = ieee80211_find_channel(ic, 
+					bestchan->ic_freq, bestchan->ic_flags &
+					~IEEE80211_CHAN_TURBO)) == NULL) {
 				/* should never happen ?? */
 				SCAN_AP_UNLOCK_IRQ_EARLY(as);
 				return 0;
 			}
 		}
 		memset(&se, 0, sizeof(se));
-		se.se_chan = c;
+		se.se_chan = bestchan;
 
 		as->as_action = ss->ss_ops->scan_default;
 		if (action)
 			as->as_action = action;
 		as->as_selbss = se;
 
-		/* 
-		 * Must defer action to avoid possible recursive call through 80211
-		 * state machine, which would result in recursive locking.
-		 */
+		/* Must defer action to avoid possible recursive call through 
+		 * 80211 state machine, which would result in recursive 
+		 * locking. */
 		IEEE80211_SCHEDULE_TQUEUE(&as->as_actiontq);
 		res = 1;
 	}
